__name__ = 'csv-pm-llm-parsing'
VERSION = '0.1'
__version__ = VERSION
__doc__ = 'LLM-based CSV parsing for Process Mining purposes'
__author__ = 'Alessandro Berti, PADS @ RWTH'
__author_email__ = 'a.berti@pads.rwth-aachen.de'
__maintainer__ = 'Alessandro Berti, PADS @ RWTH'
__maintainer_email__ = "a.berti@pads.rwth-aachen.de"
